MODULE asmphytobal_hadocc
   !!======================================================================
   !!                       ***  MODULE asmphytobal_hadocc  ***
   !! Calculate increments to HadOCC based on surface phyto increments
   !!
   !! IMPORTANT NOTE: This calls the bioanalysis routine of Hemmings et al.
   !! For licensing reasons this is kept in its own internal Met Office
   !! branch (dev/frdf/vn3.6_nitrogen_balancing) rather than in the Paris
   !! repository, and must be merged in when building.
   !!
   !!======================================================================
   !! History :  3.6  ! 2017-08  (D. Ford)     Adapted from bioanal.F90
   !!----------------------------------------------------------------------
#if defined key_asminc && defined key_hadocc
   !!----------------------------------------------------------------------
   !! 'key_asminc'          : assimilation increment interface
   !! 'key_hadocc'          : HadOCC model
   !!----------------------------------------------------------------------
   !! asm_phyto_bal_hadocc : routine to calculate increments to HadOCC
   !!----------------------------------------------------------------------
   USE par_kind,      ONLY: wp             ! kind parameters
   USE par_oce,       ONLY: jpi, jpj, jpk  ! domain array sizes
   USE dom_oce,       ONLY: gdepw_n        ! domain information
   USE iom                                 ! i/o
   USE par_hadocc                          ! HadOCC parameters
   USE had_bgc_stnd,  ONLY: kmt            ! HadOCC parameters
   USE had_bgc_const                       ! HadOCC parameters
   USE par_trc,       ONLY: jptra          ! Tracer parameters
   USE bioanalysis                         ! Nitrogen balancing

   IMPLICIT NONE
   PRIVATE                   

   PUBLIC asm_phyto_bal_hadocc

   ! Default values for biological assimilation parameters
   ! Should match Hemmings et al. (2008)
   REAL(wp), PARAMETER :: balnutext  =  0.6    !: Default nutrient balancing factor
   REAL(wp), PARAMETER :: balnutmin  =  0.1    !: Fraction of phytoplankton loss to nutrient
   REAL(wp), PARAMETER :: r          =  1      !: Reliability of model specific growth rate
   REAL(wp), PARAMETER :: beta_g     =  0.05   !: Low rate bias correction for growth rate estimator
   REAL(wp), PARAMETER :: beta_l     =  0.05   !: Low rate bias correction for primary loss rate estimator
   REAL(wp), PARAMETER :: beta_m     =  0.05   !: Low rate bias correction for secondary loss rate estimator
   REAL(wp), PARAMETER :: a_g        =  0.2    !: Error s.d. for log10 of growth rate estimator
   REAL(wp), PARAMETER :: a_l        =  0.4    !: Error s.d. for log10 of primary loss rate estimator
   REAL(wp), PARAMETER :: a_m        =  0.7    !: Error s.d. for log10 of secondary loss rate estimator
   REAL(wp), PARAMETER :: zfracb0    =  0.7    !: Base zooplankton fraction of loss to Z & D
   REAL(wp), PARAMETER :: zfracb1    =  0      !: Phytoplankton sensitivity of zooplankton fraction
   REAL(wp), PARAMETER :: qrfmax     =  1.1    !: Maximum nutrient limitation reduction factor
   REAL(wp), PARAMETER :: qafmax     =  1.1    !: Maximum nutrient limitation amplification factor
   REAL(wp), PARAMETER :: zrfmax     =  2      !: Maximum zooplankton reduction factor
   REAL(wp), PARAMETER :: zafmax     =  2      !: Maximum zooplankton amplification factor
   REAL(wp), PARAMETER :: prfmax     =  10     !: Maximum phytoplankton reduction factor (secondary)
   REAL(wp), PARAMETER :: incphymin  =  0.0001 !: Minimum size of non-zero phytoplankton increment
   REAL(wp), PARAMETER :: integnstep =  20     !: Number of steps for p.d.f. integral evaluation
   REAL(wp), PARAMETER :: pthreshold =  0.01   !: Fractional threshold level for setting p.d.f.
   !
   LOGICAL,  PARAMETER :: diag_active           = .TRUE.  !: Depth-independent diagnostics
   LOGICAL,  PARAMETER :: diag_fulldepth_active = .TRUE.  !: Full-depth diagnostics
   LOGICAL,  PARAMETER :: gl_active             = .TRUE.  !: Growth/loss-based balancing
   LOGICAL,  PARAMETER :: nbal_active           = .TRUE.  !: Nitrogen balancing
   LOGICAL,  PARAMETER :: subsurf_active        = .TRUE.  !: Increments below MLD
   LOGICAL,  PARAMETER :: deepneg_active        = .FALSE. !: Negative primary increments below MLD
   LOGICAL,  PARAMETER :: deeppos_active        = .FALSE. !: Positive primary increments below MLD
   LOGICAL,  PARAMETER :: nutprof_active        = .TRUE.  !: Secondary increments

CONTAINS

   SUBROUTINE asm_phyto_bal_hadocc( kdeps,                              &
      &                             ld_chltot,                          &
      &                             pinc_chltot_3d,                     &
      &                             ld_phytot,                          &
      &                             pinc_phytot_3d,                     &
      &                             pincper,                            &
      &                             p_maxchlinc, ld_phytobal, pmld,     &
      &                             pgrow_avg_bkg_3d, ploss_avg_bkg_3d, &
      &                             phyt_avg_bkg_3d, mld_max_bkg,       &
      &                             cchl_p_bkg_3d,                      &
      &                             tracer_bkg, phyto_balinc )
      !!---------------------------------------------------------------------------
      !!                    ***  ROUTINE asm_phyto_bal_hadocc  ***
      !!
      !! ** Purpose :   calculate increments to HadOCC from 2d phytoplankton increments
      !!
      !! ** Method  :   call nitrogen balancing scheme
      !!
      !! ** Action  :   populate phyto_balinc
      !!
      !! References :   Hemmings et al., 2008, J. Mar. Res.
      !!                Ford et al., 2012, Ocean Sci.
      !!---------------------------------------------------------------------------
      !!
      INTEGER,  INTENT(in   )                               :: kdeps            ! No. inc deps 1 or jpk
      LOGICAL,  INTENT(in   )                               :: ld_chltot        ! Assim chltot y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj,kdeps)     :: pinc_chltot_3d   ! chltot increments
      LOGICAL,  INTENT(in   )                               :: ld_phytot        ! Assim phytot y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj,kdeps)     :: pinc_phytot_3d   ! phytot increments
      REAL(wp), INTENT(in   )                               :: pincper          ! Assimilation period
      REAL(wp), INTENT(in   )                               :: p_maxchlinc      ! Max chl increment
      LOGICAL,  INTENT(in   )                               :: ld_phytobal      ! Balancing y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj)           :: pmld             ! Mixed layer depth
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,kdeps)     :: pgrow_avg_bkg_3d ! Avg phyto growth
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,kdeps)     :: ploss_avg_bkg_3d ! Avg phyto loss
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,kdeps)     :: phyt_avg_bkg_3d  ! Avg phyto
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj)           :: mld_max_bkg      ! Max MLD
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,kdeps)     :: cchl_p_bkg_3d    ! C:Chl
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,jpk,jptra) :: tracer_bkg       ! State variables
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk,jptra) :: phyto_balinc     ! Balancing increments
      !!
      INTEGER                                               :: ji, jj, jk, jn    ! Loop counters
      INTEGER                                               :: jkmax             ! Loop index
      INTEGER,                 DIMENSION(6)                 :: i_tracer          ! Tracer indices
      REAL(wp),                DIMENSION(16)                :: modparm           ! Model parameters
      REAL(wp),                DIMENSION(20)                :: assimparm         ! Assimilation parameters
      REAL(wp),                DIMENSION(jpi,jpj,1,6)       :: bstate_2d         ! Background state (2D)
      REAL(wp),                DIMENSION(jpi,jpj,jpk,6)     :: bstate_3d         ! Background state (3D)
      REAL(wp),                DIMENSION(jpi,jpj,1,6)       :: outincs_2d        ! Balancing increments (2D)
      REAL(wp),                DIMENSION(jpi,jpj,jpk,6)     :: outincs_3d        ! Balancing increments (3D)
      REAL(wp),                DIMENSION(jpi,jpj,22)        :: diag              ! Depth-indep diagnostics
      REAL(wp),                DIMENSION(jpi,jpj,1,22)      :: diag_fulldepth_2d ! Full-depth diagnostics (2D)
      REAL(wp),                DIMENSION(jpi,jpj,jpk,22)    :: diag_fulldepth_3d ! Full-depth diagnostics (3D)
      REAL(wp),                DIMENSION(jpi,jpj)           :: cchl_p_bkg_2d     ! C:Chl for total phy (2D)
      REAL(wp),                DIMENSION(jpi,jpj,1)         :: tmask_2d          ! Single-level tmask
      REAL(wp),                DIMENSION(jpi,jpj)           :: pinc_chltot_2d    ! chltot increments (2D)
      REAL(wp),                DIMENSION(jpi,jpj)           :: pinc_phytot_2d    ! phytot increments (2D)
      REAL(wp),                DIMENSION(jpi,jpj)           :: pgrow_avg_bkg_2d  ! Avg phyto growth (2D)
      REAL(wp),                DIMENSION(jpi,jpj)           :: ploss_avg_bkg_2d  ! Avg phyto loss (2D)
      REAL(wp),                DIMENSION(jpi,jpj)           :: phyt_avg_bkg_2d   ! Avg phyto (2D)
      !!---------------------------------------------------------------------------

      IF ( ( .NOT. ld_chltot ) .AND. ( .NOT. ld_phytot ) ) THEN
         CALL ctl_stop( ' Trying to do phyto balancing but nothing to assimilate' )
      ENDIF
      
      ! If p_maxchlinc > 0 then cap total absolute chlorophyll increment at that value
      IF ( p_maxchlinc > 0.0 ) THEN
         IF ( ld_chltot ) THEN
            DO jk = 1, kdeps
               DO jj = 1, jpj
                  DO ji = 1, jpi
                     pinc_chltot_3d(ji,jj,jk) = MAX( -1.0 * p_maxchlinc, MIN( pinc_chltot_3d(ji,jj,jk), p_maxchlinc ) )
                  END DO
               END DO
            END DO
         ENDIF
      ENDIF

      IF ( ld_phytot ) THEN
         CALL ctl_stop( ' No phytoplankton carbon assimilation quite yet' )
      ENDIF
      
      IF ( ld_phytobal ) THEN   ! Nitrogen balancing

         ! Set up model parameters to be passed into Hemmings balancing routine
         modparm(1)  = grow_sat
         modparm(2)  = psmax
         modparm(3)  = par
         modparm(4)  = alpha
         modparm(5)  = resp_rate
         modparm(6)  = pmort_rate
         modparm(7)  = phyto_min
         modparm(8)  = z_mort_1
         modparm(9)  = z_mort_2
         modparm(10) = c2n_p
         modparm(11) = c2n_z
         modparm(12) = c2n_d
         modparm(13) = graze_threshold
         modparm(14) = holling_coef
         modparm(15) = graze_sat
         modparm(16) = graze_max

         ! Set up assimilation parameters to be passed into balancing routine
         ! Not sure what assimparm(1) is meant to be, but it doesn't get used
         assimparm(2)  = balnutext
         assimparm(3)  = balnutmin
         assimparm(4)  = r
         assimparm(5)  = beta_g
         assimparm(6)  = beta_l
         assimparm(7)  = beta_m
         assimparm(8)  = a_g
         assimparm(9)  = a_l
         assimparm(10) = a_m
         assimparm(11) = zfracb0
         assimparm(12) = zfracb1
         assimparm(13) = qrfmax
         assimparm(14) = qafmax
         assimparm(15) = zrfmax
         assimparm(16) = zafmax
         assimparm(17) = prfmax
         assimparm(18) = incphymin
         assimparm(19) = integnstep
         assimparm(20) = pthreshold

         ! Set up external tracer indices array bstate
         i_tracer(1) = 1   ! nutrient
         i_tracer(2) = 2   ! phytoplankton
         i_tracer(3) = 3   ! zooplankton
         i_tracer(4) = 4   ! detritus
         i_tracer(5) = 5   ! DIC
         i_tracer(6) = 6   ! Alkalinity

         ! Set background state
         bstate_3d(:,:,:,i_tracer(1)) = tracer_bkg(:,:,:,jp_had_nut)
         bstate_3d(:,:,:,i_tracer(2)) = tracer_bkg(:,:,:,jp_had_phy)
         bstate_3d(:,:,:,i_tracer(3)) = tracer_bkg(:,:,:,jp_had_zoo)
         bstate_3d(:,:,:,i_tracer(4)) = tracer_bkg(:,:,:,jp_had_pdn)
         bstate_3d(:,:,:,i_tracer(5)) = tracer_bkg(:,:,:,jp_had_dic)
         bstate_3d(:,:,:,i_tracer(6)) = tracer_bkg(:,:,:,jp_had_alk)

         ! Call nitrogen balancing routine
         IF (kdeps == 1) THEN
            pinc_chltot_2d(:,:)   = pinc_chltot_3d(:,:,1)
            cchl_p_bkg_2d(:,:)    = cchl_p_bkg_3d(:,:,1)
            phyt_avg_bkg_2d(:,:)  = phyt_avg_bkg_3d(:,:,1)
            pgrow_avg_bkg_2d(:,:) = pgrow_avg_bkg_3d(:,:,1)
            ploss_avg_bkg_2d(:,:) = ploss_avg_bkg_3d(:,:,1)
            
            CALL bio_analysis( jpi, jpj, jpk, ZDZ(:,:,:), i_tracer, modparm,               &
               &               n2be_p, n2be_z, n2be_d, assimparm,                          &
               &               INT(pincper), 1, kmt(:,:), tmask(:,:,:),                    &
               &               pmld(:,:), mld_max_bkg(:,:), pinc_chltot_2d(:,:), cchl_p_bkg_2d(:,:), &
               &               nbal_active, phyt_avg_bkg_2d(:,:),                          &
               &               gl_active, pgrow_avg_bkg_2d(:,:), ploss_avg_bkg_2d(:,:),    &
               &               subsurf_active, deepneg_active,                             &
               &               deeppos_active, nutprof_active,                             &
               &               bstate_3d, outincs_3d,                                      &
               &               diag_active, diag,                                          &
               &               diag_fulldepth_active, diag_fulldepth_3d )
         ELSE
            pmld(:,:) = 0.5
            
            DO jk = 1, kdeps
               pinc_chltot_2d(:,:)   = pinc_chltot_3d(:,:,jk)
               cchl_p_bkg_2d(:,:)    = cchl_p_bkg_3d(:,:,jk)
               phyt_avg_bkg_2d(:,:)  = phyt_avg_bkg_3d(:,:,jk)
               pgrow_avg_bkg_2d(:,:) = pgrow_avg_bkg_3d(:,:,jk)
               ploss_avg_bkg_2d(:,:) = ploss_avg_bkg_3d(:,:,jk)
               tmask_2d(:,:,1)       = tmask(:,:,jk)
               bstate_2d(:,:,1,:)    = bstate_3d(:,:,jk,:)
               outincs_2d(:,:,:,:)   = 0.0

               CALL bio_analysis( jpi, jpj, 1, gdepw_n(:,:,2), i_tracer, modparm,            &
                  &               n2be_p, n2be_z, n2be_d, assimparm,                         &
                  &               INT(pincper), 1, INT(SUM(tmask_2d,3)), tmask_2d(:,:,:),    &
                  &               pmld(:,:), pmld(:,:), pinc_chltot_2d(:,:), cchl_p_bkg_2d(:,:), &
                  &               nbal_active, phyt_avg_bkg_2d(:,:),                         &
                  &               gl_active, pgrow_avg_bkg_2d(:,:), ploss_avg_bkg_2d(:,:),   &
                  &               subsurf_active, deepneg_active,                            &
                  &               deeppos_active, nutprof_active,                            &
                  &               bstate_2d, outincs_2d,                                     &
                  &               diag_active, diag,                                         &
                  &               diag_fulldepth_active, diag_fulldepth_2d )

               outincs_3d(:,:,jk,:) = outincs_2d(:,:,1,:)
            END DO
         ENDIF

         ! Save balancing increments
         phyto_balinc(:,:,:,jp_had_nut) = outincs_3d(:,:,:,i_tracer(1))
         phyto_balinc(:,:,:,jp_had_phy) = outincs_3d(:,:,:,i_tracer(2))
         phyto_balinc(:,:,:,jp_had_zoo) = outincs_3d(:,:,:,i_tracer(3))
         phyto_balinc(:,:,:,jp_had_pdn) = outincs_3d(:,:,:,i_tracer(4))
         phyto_balinc(:,:,:,jp_had_dic) = outincs_3d(:,:,:,i_tracer(5))
         phyto_balinc(:,:,:,jp_had_alk) = outincs_3d(:,:,:,i_tracer(6))
      
      ELSE   ! No nitrogen balancing
      
         ! Initialise phytoplankton increment to zero
         phyto_balinc(:,:,:,jp_had_phy) = 0.0
         
         ! Convert surface chlorophyll increment to phytoplankton nitrogen
         DO jk = 1, kdeps
            phyto_balinc(:,:,jk,jp_had_phy) = ( cchl_p_bkg_3d(:,:,jk) / (mw_carbon * c2n_p) ) * pinc_chltot_3d(:,:,jk)
         END DO
         
         IF (kdeps == 1) THEN
            ! Propagate through mixed layer
            DO jj = 1, jpj
               DO ji = 1, jpi
                  !
                  jkmax = jpk-1
                  DO jk = jpk-1, 1, -1
                     IF ( ( pmld(ji,jj) >  gdepw_n(ji,jj,jk)   ) .AND. &
                        & ( pmld(ji,jj) <= gdepw_n(ji,jj,jk+1) ) ) THEN
                        pmld(ji,jj) = gdepw_n(ji,jj,jk+1)
                        jkmax = jk
                     ENDIF
                  END DO
                  !
                  DO jk = 2, jkmax
                     phyto_balinc(ji,jj,jk,jp_had_phy) = phyto_balinc(ji,jj,1,jp_had_phy)
                  END DO
                  !
               END DO
            END DO
         ENDIF

         ! Set other balancing increments to zero
         phyto_balinc(:,:,:,jp_had_nut) = 0.0
         phyto_balinc(:,:,:,jp_had_zoo) = 0.0
         phyto_balinc(:,:,:,jp_had_pdn) = 0.0
         phyto_balinc(:,:,:,jp_had_dic) = 0.0
         phyto_balinc(:,:,:,jp_had_alk) = 0.0
      
      ENDIF

   END SUBROUTINE asm_phyto_bal_hadocc

#else
   !!----------------------------------------------------------------------
   !!   Default option : Empty routine
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE asm_phyto_bal_hadocc( kdeps,                              &
      &                             ld_chltot,                          &
      &                             pinc_chltot_3d,                     &
      &                             ld_phytot,                          &
      &                             pinc_phytot,                        &
      &                             pincper,                            &
      &                             p_maxchlinc, ld_phytobal, pmld,     &
      &                             pgrow_avg_bkg_3d, ploss_avg_bkg_3d, &
      &                             phyt_avg_bkg_3d, mld_max_bkg,       &
      &                             cchl_p_bkg_3d,                      &
      &                             tracer_bkg, phyto_balinc )
      INTEGER :: kdeps
      LOGICAL :: ld_chltot
      REAL    :: pinc_chltot_3d(:,:,:)
      LOGICAL :: ld_phytot
      REAL    :: pinc_phytot_3d(:,:,:)
      REAL    :: pincper
      REAL    :: p_maxchlinc
      LOGICAL :: ld_phytobal
      REAL    :: pmld(:,:)
      REAL    :: pgrow_avg_bkg_3d(:,:,:)
      REAL    :: ploss_avg_bkg_3d(:,:,:)
      REAL    :: phyt_avg_bkg_3d(:,:,:)
      REAL    :: mld_max_bkg(:,:)
      REAL    :: cchl_p_bkg_3d(:,:,:)
      REAL    :: tracer_bkg(:,:,:,:)
      REAL    :: phyto_balinc(:,:,:,:)
      WRITE(*,*) 'asm_phyto_bal_hadocc: You should not have seen this print! error?'
   END SUBROUTINE asm_phyto_bal_hadocc
#endif

   !!======================================================================
END MODULE asmphytobal_hadocc
